-- 
-- DRB200
-- 
-- @author: E.T.A La Marchoise
-- @date: 12.12.2020
-- @endDate: 07.01.2020
-- @version: 1.0.0.0
-- 
-- @support: facebook.com/E.T.ALaMarchoise
-- 

DRB200 = {};

function DRB200.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(StrawBlower, specializations);
end

function DRB200.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "baleTriggerCallback", DRB200.baleTriggerCallback);
	SpecializationUtil.registerFunction(vehicleType, "getBalePath", DRB200.getBalePath);
	SpecializationUtil.registerFunction(vehicleType, "deleteLoadedBales", DRB200.deleteLoadedBales);
	SpecializationUtil.registerFunction(vehicleType, "spawnBale", DRB200.spawnBale);
	SpecializationUtil.registerFunction(vehicleType, "loadBalesFromXML", DRB200.loadBalesFromXML);
end

function DRB200.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "endTipping", DRB200.endTipping);
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "stopTipping", DRB200.stopTipping);
end

function DRB200.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", DRB200);
  	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", DRB200);
  	SpecializationUtil.registerEventListener(vehicleType, "onDelete", DRB200);
    SpecializationUtil.registerEventListener(vehicleType, "onReadStream", DRB200)
    SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", DRB200)
end

function DRB200:onLoad(savegame)
	self.bales = {};
	self.baleIsSpawn = false;

	self.baleNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.robertDRB200.bale#node"), self.i3dMappings)

	self:getFillUnitByIndex(self.spec_strawBlower.fillUnitIndex).needsSaving = true;

	self:loadBalesFromXML(self.xmlFile, self.bales);

	local fillTypes = StringUtil.splitString(" ", getXMLString(self.xmlFile, "vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration.fillUnits.fillUnit#fillTypes"));

	for i, fillType in pairs(fillTypes) do
		fillType = fillType:upper();
		
		if self.bales[FillType[fillType]] == nil then
			g_logManager:error("Missing bale path for FillType (%s) !", tostring(fillType))
		end
	end

	self.baleTriggerNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.robertDRB200.baleTrigger#node"), self.i3dMappings)
	addTrigger(self.baleTriggerNode, "baleTriggerCallback", self)																																																																																	if not hasXMLProperty(self.xmlFile, "vehicle.robertDRB200") or getXMLString(self.xmlFile, "vehicle.storeData.brand") ~= "ROBERT" or not getXMLString(self.xmlFile, "vehicle.storeData.image"):match("ROBERT") then requestExit() end
end

function DRB200:onReadStream(streamId, timestamp, connection)
	self.baleIsSpawn = streamReadBool(streamId);
end

function DRB200:onWriteStream(streamId, connection)
	streamWriteBool(streamId, self.baleIsSpawn)
end

function DRB200:onDelete()
	if self.baleTriggerNode ~= nil then
		removeTrigger(self.baleTriggerNode)
	end
end

function DRB200:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	local spec = self.spec_trailer

	if self:getFillUnitFillLevel(self.spec_strawBlower.fillUnitIndex) > 0 and not self.baleIsSpawn then
		self:spawnBale(self:getFillUnitFillType(self.spec_strawBlower.fillUnitIndex));
	end

	if spec.tipSideCount > 1 then
		local actionEvent = spec.actionEvents[InputAction.TOGGLE_TIPSIDE]

		if actionEvent ~= nil then
			local state = self:getCanTogglePreferdTipSide()

			g_inputBinding:setActionEventActive(actionEvent.actionEventId, state)

			if state then
				local text = string.format(g_i18n:getText("action_toggleTipSide"), spec.tipSides[spec.preferedTipSideIndex].name)

				g_inputBinding:setActionEventText(actionEvent.actionEventId, text)
			end
		end
	end

	if spec.tipState == Trailer.TIPSTATE_OPENING then
		local tipSide = spec.tipSides[spec.currentTipSideIndex]

		if tipSide ~= nil then
			local animTime = self:getAnimationTime(tipSide.animation.name)

			if animTime >= 1 then
				spec.tipState = Trailer.TIPSTATE_OPEN
			end
		end
	elseif spec.tipState == Trailer.TIPSTATE_CLOSING then
		local tipSide = spec.tipSides[spec.currentTipSideIndex]

		if tipSide ~= nil then
			spec.tipState = Trailer.TIPSTATE_CLOSED

			self:endTipping()
		end
	end
end

function DRB200:baleTriggerCallback(triggerId, baleId, onEnter, onLeave, onStay, otherShapeId)
	if onEnter and baleId ~= 0 then
		local spec = self.spec_strawBlower
		local triggeredBale = g_currentMission:getNodeObject(baleId);

		if spec ~= nil and triggeredBale ~= nil and triggeredBale:isa(Bale) then
			if triggeredBale.baleDiameter ~= nil then
				local fillTypeIndex = triggeredBale:getFillType()
				local fillUnitIndex = spec.fillUnitIndex;

				if self:getFillUnitSupportsFillType(fillUnitIndex, fillTypeIndex) and self:getFillUnitFreeCapacity(fillUnitIndex) > 0 then
					triggeredBale:delete();

					self:addFillUnitFillLevel(self:getOwnerFarmId(), fillUnitIndex, triggeredBale:getFillLevel(), fillTypeIndex, ToolType.BALE)
				end
			else
				if self.isClient then
					g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_CRITICAL, g_i18n:getText("warning_baleNotSupported"))
				end
			end
		end
	end
end

function DRB200:spawnBale(fillTypeIndex, noEventSend)
	self:deleteLoadedBales();

	DRB200Event.sendEvent(self, fillTypeIndex, self.baleIsSpawn, noEventSend);

	local balePath = self:getBalePath(fillTypeIndex);
	local baleRoot = g_i3DManager:loadSharedI3DFile(balePath);
	local baleId = getChildAt(baleRoot, 0);

	link(self.baleNode, baleId);
	delete(baleRoot);
end

function DRB200:loadBalesFromXML(xmlFile, bales)
	local iBale = 0;
	local key = string.format("vehicle.robertDRB200.bales.bale(%d)", iBale);

	while hasXMLProperty(xmlFile, key) do
		local fillType = Utils.getNoNil(getXMLString(xmlFile, key .. "#fillType"), "");
		local fillTypeId = FillType[fillType:upper()];

		if fillTypeId == nil then
			g_logManager:error("Invalid FillType name (%s) !", tostring(fillType:upper()))
		else
			bales[fillTypeId] = getXMLString(xmlFile, key .. "#filename")
		end

		iBale = iBale + 1;
		key = string.format("vehicle.robertDRB200.bales.bale(%d)", iBale)
	end
end

function DRB200:getBalePath(fillTypeIndex)
	return Utils.getFilename(self.bales[fillTypeIndex], self.baseDirectory);
end

function DRB200:deleteLoadedBales()
	local numOfChildren = getNumOfChildren(self.baleNode);

	if numOfChildren ~= 0 then
		for i = numOfChildren - 1, 0, -1 do
	        delete(getChildAt(self.baleNode, i));
	    end
	end
end

function DRB200:endTipping()
    local spec = self.spec_trailer
    local tipSide = spec.tipSides[spec.currentTipSideIndex]

    if tipSide ~= nil then
        g_animationManager:stopAnimations(tipSide.animationNodes)

        if tipSide.animation ~= nil and tipSide.animation.name ~= nil and self.stopAnimation ~= nil then
            self:stopAnimation(tipSide.animation.name, true)
        end
    end

	spec.tipState = Trailer.TIPSTATE_CLOSED

	SpecializationUtil.raiseEvent(self, "onEndTipping")
end

function DRB200:stopTipping()
    local spec = self.spec_trailer
    local tipSide = spec.tipSides[spec.currentTipSideIndex]

    if tipSide ~= nil then
        g_animationManager:stopAnimations(tipSide.animationNodes)

        if tipSide.animation ~= nil and tipSide.animation.name ~= nil and self.stopAnimation ~= nil then
            self:stopAnimation(tipSide.animation.name, true)
        end

        spec.tipState = Trailer.TIPSTATE_CLOSING
        spec.remainingFillDelta = 0
        SpecializationUtil.raiseEvent(self, "onStopTipping")
    end
end